# Contributing to Lektor

Clone the [git repository](https://git.iiens.net/martin2018/lektor). Create a
branch from master with the `git checkout -b dev-${pseudal}` command.

Please, rebase regularly your branch on `master`.

Please, add your name in the CONTRIBUTORS file. The format is `First name
'Pseudal' FAMILY　NAME`. The licence is a modified version of the ISC licence
to include this file (in short, a kind of MIT licence).

## License

See [LICENSE](LICENSE).

## Building

It is recommended to use the `../configure` method to build while in
developpement. Also, prefere useing the `--with-depends` with the
configuration, to use the fixed version.

An archlinux package exists, use `makepkg -si` to build and install it. If
there are no package for your distribution, feel free to add a way to make it.

## Style and the code

### C code and style

Prefer using `__${name}` names for static variables and functions. Prefix the
name of the file or folder in function names, e.g. `database_` or `command_`.

For formating, use the `./scripts/astyle.bash` from the root of the project.
You'll need [astyle](http://astyle.sourceforge.net/) for that. Note that the
astyle step is a part of the CI/CD.

You are encouraged to use the `./scripts/tags.bash` script to build the `ctags`
and `cscope` database files to explore more easelly the project.

The C standard used here is C11, because on debian C18 doesn't compiles.

### Scripts

Scripts goes to the [scripts](scripts) folder. They must be written to be
launched from the root of the project. Include the language used in the name of
the script, for example:

- `astyle.bash`: written in bash
- `magic.rb`: written in ruby

Please, do not write scripts in too many different languages to reduce the
number of dependencies. If you are using binaries in your scripts (like astyle
or bc), include them in the binary dependencies in the [README](README.md)
file.

There is a strong possibility that your script will do similar things to what a
script already in this repo do. In that case, please work on the existing
script and enhance it with your changes. No need to duplicate things.

## Merge request titles

Try to be as more descriptive as you can in your Merge Request title. Include
the issue number and the target OS if possible.

- Branch from the master branch and, if needed, rebase to the current master
  branch before submitting your merge request. If it doesn't merge cleanly with
  master you may be asked to rebase your changes
- Commits should be as small as possible, while ensuring that each commit is
  correct independently (i.e., each commit should compile and pass tests)
- Add tests relevant to the fixed bug or new feature. In general, add a way to
  test lektor

When adding new fetures or solving issues / bugs, please edit the
[CHANGELOG](CHANGELOG) file at the `# Next` section. This will help people know
what has been done since the last version
